#----------------------------------------------------------------
# Copyright (c) 2004 Benjamin Crowell, all rights reserved.
#
# This software is available under version 2 of the GPL license.
# The software is copyrighted, and you must agree to the
# license in order to have permission to copy it. The full
# text of the license is given in the file titled Copying.
#
#----------------------------------------------------------------

use strict;

package Activity;
=head1 Activity class

This is an abstract class, which is subclassed to make more specific types of activities, e.g., an
activity for recognizing intervals, or relating a note to the tonic.

=cut

=head2 new

Activity->new(SERVER=>$server,TIMBRES=>{'accomp'=>$piano,'melody'=>$sax})

=cut

sub new {
  my $class = shift;
  my $self = {};
  my %args = (
    SERVER=>[],
    TIMBRES=>{},
		@_,
	);
  bless($self,$class);
  $self->{SERVER} = $args{SERVER};
  $self->{TIMBRES} = $args{TIMBRES};
  $self->{FSM} = undef;
  $self->{EXERCISE} = {};
  my $volume = 1;
  $self->set_volume_ref(\$volume);
  return $self;
}

=head2 play

$activity->play($audio)

Plays it with a gain set by the volume we maintain.

=cut

sub play {
  my $self = shift;
  my $audio = shift;
  $self->{SERVER}->play($audio,$self->get_volume_private_method);
}

=head2 set_volume_ref

Pass it a reference to a scalar that sets the volume (on a scale of 0 to 1).
We only maintain a reference, not a copy of its value, so if the volume changes
later, we automatically use the new, correct volume.

=cut

sub set_volume_ref {
  my $self = shift;
  $self->{VOLUME_REF} = shift;
}

=head2 get_volume_private_method

This is a private method, because we don't want anyone else storing these volume
values and then not realizing they've changed later.

=cut

sub get_volume_private_method {
  my $self = shift;
  my $ref = $self->{VOLUME_REF};
  return $$ref;
}


1;
