#!/usr/bin/perl

use strict;

use Audio::Data;
use Audio::Play;

use Synthesize;
use Music;
use FiniteStateMachine;
use Activity;
use NontonalActivity;

package ExChord;

sub do {
  my $server = shift;
  my $timbre = shift;
  my $timbre2 = shift;

  my @chord_types =  ('maj',          'maj7',         '7',      'm',       'm7',       'm7b5',             'o7'); # order in which to present them
  my %type_weights = ('maj'=>10,      'maj7'=>10,     '7'=>10,  'm'=>10,   'm7'=>10,   'm7b5'=>10,         'o7'=>10);
  my %chord_input =  ('M'=>'maj',     'M7'=>'maj7',   '7'=>'7', 'm'=>'m',  'm7'=>'m7', 'h'=>'m7b5',        'o'=>'o7');
  my %chord_names =  ('maj'=>'major', 'maj7'=>'maj7', '7'=>'7', 'm'=>'m',  'm7'=>'m7', 'm7b5'=>'half-dim', 'o7'=>'o7');
  my $type;

  my $activity = NontonalActivity->new(SERVER=>$server,TIMBRES=>{'accomp'=>$timbre,'melody'=>$timbre2});
  my $exercise = {};
  $activity->{FSM}->register({
    construct=>sub {
      my $root = Util::random_integer_in_range(50,60);
      $type = Util::random_array_element_weighted(\%type_weights);
      my $chord = Util::random_voicing($root,$type,'5');
      $exercise->{ROOT} = $root;
      $exercise->{CHORD} = $chord;
      $exercise->{SOUND} = $activity->{TIMBRES}->{'accomp'}->synthesize(CHORDS=>[$chord,],RHYTHM=>[1,],VOLUME=>1./5.);
      $exercise->{ROOT_SOUND} = $activity->{TIMBRES}->{'accomp'}->synthesize(CHORDS=>[$root,],RHYTHM=>[1,],VOLUME=>1./5.);
      $activity->{DESCRIBE_CHORD} = {
                              'maj'=>'major','m'=>'minor','maj7'=>'major 7th','7'=>'dominant seventh',
                              'm7'=>'minor 7th','mmaj7'=>'tonic minor','m7b5'=>'half-diminished',
                            }->{$type};
      $activity->play($exercise->{SOUND});
    },
    replay=>sub {
      $activity->play($exercise->{SOUND});
    },
    root=>sub {
      $activity->play($exercise->{ROOT_SOUND});
    },
  });

  my @choices = ();
  for (my $i=0; $i<=$#chord_types; $i++) {
    my $type = $chord_types[$i];
    my $input;
    foreach my $in(keys %chord_input) {
      $input = $in if $chord_input{$in} eq $type;
    }
    push @choices,($input.'='.$chord_names{$type});
  }

  # subs that depend on the user interface:
  my $print_prompt = sub {
      my $state = shift;
      print "Enter ";
      if ($state eq 'waiting') {
        print "a chord quality (".join(' ',@choices)."), or ";
      }
      print "? to hear it again, or 1 to hear the root by itself,\n";
      if ($state eq 'incorrect') {
        print "or hit enter to go on, ";
      }
      print "or q to quit this exercise\n";
    };
  my $describe_voicing = sub {
    print "voicing was ";
    my @p = $exercise->{CHORD}->pitches();
    foreach my $p(@p) {print Util::note_to_crowell($p-$exercise->{ROOT})}
    print "\n";
  };
  $activity->{FSM}->register({
	  waiting=>sub {&$print_prompt('waiting')},
		incorrect=>sub {&$print_prompt('incorrect')},
    describe=>sub {},
  });

  $activity->{FSM}->start;

  for (;;) {
    my $state = $activity->{FSM}->state;
    last if ($state eq 'end');
    my $poss = $activity->{FSM}->list_transitions;
    my $input = <STDIN>;
    chomp $input;
    my $menu = {'?'=>'replay','1'=>,'root',''=>'next','q'=>'quit'}->{$input};
    if ($menu && exists $poss->{$menu}) {
      $activity->{FSM}->transition($menu);
    }
    if (!$menu && $state eq 'waiting') {
      $input = $chord_input{$input} if exists $chord_input{$input};
      print "guess=$chord_names{$input}\n";
      if ($input eq $type) {
        print "correct\n";
        &$describe_voicing;
        $activity->{FSM}->transition('next');
      }
      else {
        print "incorrect, the chord was a ".$chord_names{$type}."\n";
        &$describe_voicing;
        $activity->{FSM}->transition('incorrect');
      }
    }
  }

}

1;
