#!/usr/bin/perl

use strict;

use Audio::Data;
use Audio::Play;

use Synthesize;
use Music;
use FiniteStateMachine;
use Activity;
use TonalActivity;

package ExInterval;

sub do {
  my $server = shift;
  my $timbre = shift;
  my $timbre2 = shift;
  my $activity = TonalActivity->new(SERVER=>$server,TIMBRES=>{'accomp'=>$timbre,'melody'=>$timbre2},SAME_KEY_PROB=>.8);
  my $exercise = $activity->{EXERCISE};
  my $max_interval = 12;
  my $n_done_in_this_tonality = 0;
  my $n_left_before_cadence = 0;
  my ($old_note1,$old_note2,$old_tonic,$old_mode);
  $activity->{FSM}->register({
    construct=>sub {
      my @scale = $activity->random_scale_as_array();
      my $timbre = $activity->{TIMBRES}->{'melody'};
      my ($diff,$note1,$note2);
      do {
        do {
          $diff = 1+int(abs(rand(19)-rand(19)));
        } while ($diff>$max_interval);
        ($note1,$note2) = random_notes_forming_interval($diff%12,\@scale);
        $note2 = $note1+$diff; # if it's compound, boost note2 up by the appropriate number of octaves
        # Up until now, they were guaranteed to have an ascending interval (note1<note2), but now remove that constraint:
        ($note1,$note2) = ($note2,$note1) if rand()<.5;
        ($note1,$note2) = Util::random_octave_transposition_list([$note1,$note2],$timbre->best);
	    } while ($note1==$old_note1 && $note2==$old_note2);
      $exercise->{NOTE1} = $note1;
      $exercise->{NOTE2} = $note2;
      ($old_note1,$old_note2) = ($note1,$note2);
      my $mid = int(($note1+$note2)/2.);
      $exercise->{TONIC} = Util::bring_within_octave($exercise->{TONIC},$mid);
      $exercise->{NOTE_SOUND} = $timbre->synthesize(CHORDS=>[Chord->new($note1),Chord->new($note2)],RHYTHM=>[1,1],VOLUME=>1.);
      if ($n_left_before_cadence<=0 || $exercise->{TONIC}!=$old_tonic || $exercise->{MODE} ne $old_mode) {
        $activity->play_cadence();
        $n_left_before_cadence = 4;
        $old_tonic = $exercise->{TONIC};
        $old_mode = $exercise->{MODE};
  	  }
      else {
        --$n_left_before_cadence;
      }
      $activity->play($exercise->{NOTE_SOUND});
    },
    replay=>sub {
      $activity->play($exercise->{NOTE_SOUND});
    },
    thing=>sub {
      $activity->play($exercise->{NOTE_SOUND});
    },
  });

  # subs that depend on the user interface:
  my $print_prompt = sub {
      my $state = shift;
      print "enter ";
      if ($state eq 'waiting') {
        print "enter interval (".Util::list_interval_names()."), or ";
      }
      print "? to hear it again, or c, or 1 to hear the cadence or tonic, ";
      if ($state eq 'incorrect') {
        print "or hit enter to go on, ";
      }
      print "or q to quit this exercise\n";
    };
  $activity->{FSM}->register({
	  waiting=>sub {&$print_prompt('waiting')},
		incorrect=>sub {&$print_prompt('incorrect')},
  });

  $activity->{FSM}->start;

  for (;;) {
    my $state = $activity->{FSM}->state;
    last if ($state eq 'end');
    my $poss = $activity->{FSM}->list_transitions;
    my $input = <STDIN>;
    chomp $input;
    my $menu = {'?'=>'replay','1'=>,'tonic','n'=>'thing','c'=>'cadence',''=>'next','q'=>'quit'}->{$input};
    if ($menu && exists $poss->{$menu}) {
      $activity->{FSM}->transition($menu);
    }
    if (!$menu && $state eq 'waiting') {
      my $guess = Util::name_to_interval($input);
      if (! defined $guess) {
        print "Invalid input: $input\n";
      }
      else {
        print "guess=".Util::interval_to_name($guess)."\n";
        my ($note1,$note2,$tonic) = ($exercise->{NOTE1},$exercise->{NOTE2},$exercise->{TONIC});
        my $actual = abs($note2-$note1);
        my $describe = Util::note_to_crowell($note1-$tonic).' '.Util::note_to_crowell($note2-$tonic);
        if ($guess==$actual) {
          print "correct ($describe)\n";
          $activity->{FSM}->transition('next');
        }
        else {
          print "incorrect, the interval was ".Util::interval_to_name($actual)." ($describe)\n";
          $activity->{FSM}->transition('incorrect');
        }
		  }
    }
  }

}

# pick two random notes from the given scale that form the given interval

sub random_notes_forming_interval {
  my $diff = shift;
  my $scale = shift;
  my @scale = @$scale;
  my $n = $#scale;
  # Search for a combination of notes that will make the desired interval. This algorithm
  # only works if it's less than an octave.
  # j=note in scale (0=tonic, 1=2nd,...)
  # k=another note in scale
  # inv=whether to invert them (otherwise we never get intervals that leap across the tonic)
  my @possibilities = ();
  for (my $j=0; $j<=$n; $j++) {
    for (my $k=0; $k<=$j; $k++) { # don't double-count; allow k==j in case they want unisons
      for (my $inv=0; $inv<=1; $inv++) {
        my $note2 = @scale[$j]; # higher note
        my $note1 = @scale[$k]; # lower note
        if ($inv) {
          ($note1,$note2) = ($note2,$note1+12);
        }
        if(abs($note1-$note2)==$diff) {
          push @possibilities,[$note1,$note2];
        }
      } # end loop over inv
    } # end loop over k
  } # end loop over j
  if (!@possibilities) {
    die "error picking notes for the interval of $diff half-steps"; # could happen if it's a symmetric scale: diminished, wholetone, ...
  }
  my $possibility = Util::random_array_element(@possibilities);
  my ($note1,$note2) = ($possibility->[0],$possibility->[1]);


  return ($note1,$note2);

}


1;
