#!/usr/bin/perl

use strict;

use Audio::Data;
use Audio::Play;

use Synthesize;
use Music;
use Melody;
use Clone;

package ExMelody;

sub do {
  my $server = shift;
  my $timbre = shift;
  my $timbre2 = shift;
  my $melodies_dir = shift;
  my $n_notes = shift; # 1=note, 2=interval

  my $activity = TonalActivity->new(SERVER=>$server,TIMBRES=>{'accomp'=>$timbre,'melody'=>$timbre},SAME_KEY_PROB=>.8,SET_KEY_MYSELF=>0);
  my $exercise = $activity->{EXERCISE};

  push @Melody::ISA, 'Clone'; # ------------------ bug ----------- obviously shouldn't do this every time

  my $what = ($n_notes==1 ? 'note' : 'interval');

  my ($whole_sound,$slow_sound);
  my $volume = .5;

  $activity->{FSM}->register({
    construct=>sub {
      my ($melody,$partial_melody,$i,$tonality_firmly_established,$transposition,$tempo_slow);
      do {
        ($melody,$partial_melody,$i,$tonality_firmly_established,$tempo_slow) = choose_melody_ex($melodies_dir,$what,$n_notes);
        my ($lo,$hi) = $melody->lo_and_hi;
      	$transposition = $timbre->random_transposition($lo,$hi);
      } while (! defined $transposition); # only happens if range of melody is too wide to play, which should never actually happen
      $exercise->{NEED_CADENCE} = $what eq 'note' && !$tonality_firmly_established;
      $melody->transpose($transposition);
      $partial_melody->transpose($transposition);
      $activity->set_tonic_and_mode($melody->tonic,$melody->mode);
      $exercise->{MELODY} = $melody;
      print $exercise->{MELODY}->description . "\n"; # -------------------- bug ---- depends on user interface, shouldn't be in here
      $exercise->{PARTIAL_MELODY} = $partial_melody;
      $exercise->{NOTE} = $i;
      $exercise->{SOUND} = $partial_melody->synthesize($timbre,$volume*2,$tempo_slow);
      $exercise->{WHOLE_SOUND} = undef;
      $exercise->{SLOW_SOUND} = undef;
      if ($exercise->{NEED_CADENCE}) {
        $activity->play_cadence();
    	}
      $activity->play($exercise->{SOUND});
    },
    replay=>sub {
      $activity->play($exercise->{SOUND});
    },
    thing=>sub {
      $activity->play($exercise->{SOUND});
    },
    whole=>sub {
      if (!defined $exercise->{WHOLE_SOUND}) {
        $exercise->{WHOLE_SOUND} = $exercise->{MELODY}->synthesize($timbre,$volume,$exercise->{MELODY}->tempo);
      }
      $activity->play($exercise->{WHOLE_SOUND});
    },
    half_speed=>sub {
      if (!defined $exercise->{SLOW_SOUND}) {
        $exercise->{SLOW_SOUND} = $exercise->{PARTIAL_MELODY}->synthesize($timbre,$volume,$exercise->{MELODY}->tempo/2.);
      }
      $activity->play($exercise->{SLOW_SOUND});
    },
  });

  my $print_prompt = sub {
		my $state = shift;
    my $describe;
    print "Enter ";
		if ($state eq 'waiting') {
      if ($what eq 'note') {
        $describe = 'solfeg letter ('.Util::list_crowell_names().')';
      } 
      else {
        $describe='interval ('.Util::list_interval_names().')';
      }
      print "$describe, or ";
  	}
    print "? to hear the example again,\n";
    print "or w to hear the whole melody, or h to play it at half the tempo,\n";
    print "or c or 1 to hear the cadence or tonic, or q to quit this exercise.\n";
		if ($state eq 'incorrect') {
			print "or hit enter to go on, ";
		}
		print "or q to quit this exercise\n";
	};
  $activity->{FSM}->register({
    waiting=>sub {&$print_prompt('waiting')},
    incorrect=>sub {&$print_prompt('incorrect')},
  });

  $activity->{FSM}->start;

  for (;;) {
    my $state = $activity->{FSM}->state;
    last if ($state eq 'end');
    my $poss = $activity->{FSM}->list_transitions;
    my $input = <STDIN>;
    chomp $input;
    my $menu = {'?'=>'replay','1'=>,'tonic','n'=>'thing','c'=>'cadence',''=>'next','q'=>'quit','w'=>'whole','h'=>'half_speed'}->{$input};
    if ($menu && exists $poss->{$menu}) {
      $activity->{FSM}->transition($menu);
    }
    if (!$menu && $state eq 'waiting') {
      my ($guess,$actual);
      my $describe = '';
      if ($what eq 'note') {
        $actual = ($exercise->{MELODY}->notes->[$exercise->{NOTE}]-$exercise->{MELODY}->tonic)%12;
        $guess = Util::crowell_to_note($input);
        print "guess=".Util::note_to_crowell($guess)."\n";
      }
      else {
        $actual = abs($exercise->{MELODY}->notes->[$exercise->{NOTE}]-$exercise->{MELODY}->notes->[$exercise->{NOTE}+1]);
        $guess = Util::name_to_interval($input);
        print "guess=".Util::interval_to_name($guess)."\n";
        if ($what eq 'interval') {
          $describe = ' ('.Util::note_to_crowell($exercise->{MELODY}->notes->[$exercise->{NOTE}]-$exercise->{MELODY}->tonic).' '.Util::note_to_crowell($exercise->{MELODY}->notes->[$exercise->{NOTE}+1]-$exercise->{MELODY}->tonic).')';
        }
      }
      if ($guess==$actual) {
        print "correct$describe\n";
        $activity->{FSM}->transition('next');
      }
      else {
        print "incorrect, the $what was ".
                              ($what eq 'note' ? Util::note_to_crowell($actual) : Util::interval_to_name($actual))."$describe\n";
        $activity->{FSM}->transition('incorrect');
      }
    }
  }
}



sub choose_melody_ex {
    my $melodies_dir = shift;
    my $what = shift;
    my $n_notes = shift;

    my $contrast = 11.; # how many times louder to make the note they're supposed to hear out
    my $melody = Melody->random($melodies_dir);
    my $bar = $melody->{DATA}->{bar};
    my $rhythm = $melody->{DATA}->{rhythm};
    my $properties = $melody->{DATA}->{properties};
    my $meta = $melody->{DATA}->{meta};
    my $i; # which notes to use -- make i and i+1 louder
    my @possible_notes; # list possibilities; don't use rests or repeated notes, or very short notes
    my $tempo_slow = ($melody->tempo)/4.; # List all the possibilities with a relatively slow tempo first. Otherwise, a melody with 2 long notes always uses those.
    my $min_length = .15; # This is essentially the shortest note length we allow (in seconds), but see below for exceptions.
    my $n_tries = 0;
    while ($n_tries++<100) { # Loop until we get a possible choice. If we fail, we slow down the tempo on the next time through.
      @possible_notes = ();
      for (my $i=0; $i<=$melody->length-$n_notes; $i++) {
        my $ok = 1;
        for (my $j=0; $j<$n_notes; $j++) {
          $ok = $ok && defined $melody->notes->[$i+$j];
          $ok = $ok && $rhythm->[$i+$j]/$tempo_slow>$min_length;
          if ($ok && $j==1) {$ok = $ok && $melody->notes->[$i+$j]!=$melody->notes->[$i+$j-1]}
			  }
        push @possible_notes,$i if $ok;
      }
      last if @possible_notes;
      $tempo_slow *= 0.5;
	  }
    die "unable to find notes to use" if !@possible_notes;
    my $i = Util::random_array_element(@possible_notes); # make notes i and i+1 louder
    # Now speed it back up, if possible. (This is necessary because we start with tempo/4 above.)
    my $f = 1.2; # factor by which to speed it up each time
    if ($what eq 'note') {
      for (;;) {
        my $l = $rhythm->[$i]/$tempo_slow*$f;         # The proposed new, shorter lengths (in seconds):
        last unless ($tempo_slow*$f<=$melody->tempo && $l>$min_length);
        $tempo_slow *= $f;
      }
	  }
    if ($what eq 'interval') {
      for (;;) {
        # The proposed new, shorter lengths (in seconds):
        my $l1 = $rhythm->[$i]/$tempo_slow*$f;
        my $l2 = $rhythm->[$i+1]/$tempo_slow*$f;
        # It's actually not so bad to have one of the notes be short, as long as the other one is nice and long.
        ($l1,$l2) = ($l2,$l1) if $l1>$l2; # Make l1 the shorter one.
        last unless ($tempo_slow*$f<=$melody->tempo && $l1>$min_length/4 && $l2>$min_length && ($l1+$l2)>$min_length*2);
        $tempo_slow *= 1.2;
      }
	  }

    my $desired_time = rand()*3+3; # number of seconds we want the whole melodic fragment to last, roughly
    my $max_time = 10.; # max seconds if necessary in order to establish tonality
    my $desired_whole = $desired_time * $tempo_slow; # desired number of whole notes
    my $max_whole = $max_time * $tempo_slow;

    my ($start,$end) = ($i,$i+$n_notes-1);
    my $time = 0; # in units of whole notes
    for (my $j=0; $j<$n_notes; $j++) {
      $time += $rhythm->[$i]; 
    }
    # Lengthen the snippet of melody until it's as long as we wanted it to be. Randomly choose to add another note
    # on either the beginning or the end. Rinse, lather, repeat.
    my $min_notes = 5;
    if ($what eq 'note') {
      $min_notes = 8; # need more to establish tonality
    }
    # For note recognition, we want enough different notes to define the tonality reasonably well.
    my %used_notes = ();
    for (my $k=$start; $k<=$end; $k++) {
      if (defined $melody->notes->[$k]) {
        $used_notes{($melody->notes->[$k]-$melody->tonic)%12} = 1;
      }
    }
    my $tonality_firmly_established = 0;
    for (;;) { # always at least do head of loop to make sure we decide if tonality
               # has been established
      my $n_used = (keys %used_notes);
      ++$n_used;
      $tonality_firmly_established 
            =    (defined $used_notes{11}) # 7 
              && (defined $used_notes{5})  # 4
              && $n_used>=3 
              && !(defined $used_notes{10}) # b7
              && !(defined $used_notes{6}); # #4
      last if ! ($start>=1 || $end<=$melody->length-2);
      my $n_included = $end-$start+1;
      last unless (
                  $time<$desired_whole
							 or     $what eq 'note' 
                  && (
                          $n_included<$min_notes
                       || ($time<$max_whole && !$tonality_firmly_established)
                     )
      );
      my $do_start = (rand()<.5);
      if ($start==0) {$do_start=0}
      if ($end==$melody->length-1) {$do_start=1}
      if ($do_start) {
        --$start;
        $time = $time+$rhythm->[$start];
        $used_notes{($melody->notes->[$start]-$melody->tonic)%12} = 1;
      }
      else {
        ++$end;
        $time = $time+$rhythm->[$end];
        $used_notes{($melody->notes->[$end]-$melody->tonic)%12} = 1;
      }
    }
    ++$start if ! defined $melody->notes->[$start]; # don't start on a rest -- should be unnecessary because of new phrase heuristics...


    my @phrase_scores = Util::phrase_score($melody->timesig,$melody->notes,$rhythm,$bar);
    my $new_start = $start;
    my $dt = 0.;
    for (my $i=$start-1; $i>=0; $i--) {
      $dt += $rhythm->[$i];

      if ($phrase_scores[$i]>$phrase_scores[$new_start] && ($dt<=$melody->timesig || $bar->[$new_start]<0.)) {
  			$new_start = $i;
      }
    }
    $start = $new_start;
    my $partial_melody = $melody->slice($start,$end);
    $partial_melody->amplify(1./$contrast);
    for (my $k=0; $k<$n_notes; $k++) {
      $partial_melody->{DATA}->{volumes}->[$i-$start+$k] *= $contrast;
    }

    return ($melody,$partial_melody,$i,$tonality_firmly_established,$tempo_slow);

}




1;
