#!/usr/bin/perl

use strict;

use Audio::Data;
use Audio::Play;

use Synthesize;
use Music;
use FiniteStateMachine;
use Activity;
use NontonalActivity;

package ExNoteChord;

sub do {
  my $server = shift;
  my $timbre = shift;
  my $timbre2 = shift;
  my $activity = NontonalActivity->new(SERVER=>$server,TIMBRES=>{'accomp'=>$timbre,'melody'=>$timbre2});
  my $exercise = {};
  my $difficulty = 5; # must be from 1 to 10
  $activity->{FSM}->register({
    construct=>sub {
      my $root = Util::random_integer_in_range(50,60);
      my %type_weights = ('maj'=>10,'7'=>10,'m'=>10,'maj7'=>7,'m7'=>7,'mmaj7'=>3,'m7b5'=>3);
      foreach my $type(keys %type_weights) {
        my $w = $type_weights{$type};
        $w = ($w+$difficulty-8);
        if ($w<0) {$w=0}
        $w = $w**(3./$difficulty);
        $type_weights{$type} = $w;
      }
      my $type = Util::random_array_element_weighted(\%type_weights);
      my $chord = Util::random_voicing($root,$type,'5');
      my $note;
      my @possible_notes;
      my $done = 0;
      my $exoticness_table = {
               #   1       2       3   4       5       6  b7  n7
      'maj'   =>[  0,  5,  0,  4,  0,  0,  3,  0,  6,  0,  0,  0],
      'maj7'  =>[  0,  5,  0,  4,  0,  0,  3,  0,  6,  0, 10,  0],
      '7'     =>[  0,  3,  0,  3,  0,  0,  3,  0,  5,  0,  0, 10],
      'm'     =>[  0,  5,  0,  0, 10,  0,  3,  0,  4,  0,  0,  5],
      'm7'    =>[  0,  5,  0,  0, 10,  0,  3,  0,  4,  0,  0, 10],
      'mmaj7' =>[  0,  5,  0,  0, 10,  0,  3,  0,  4,  0, 10,  0],
      'm7b5'  =>[  0,  0,  0,  0,  6,  0,  0, 10,  0,  0,  0, 10],
      };
      my $exoticness = $exoticness_table->{$type};
      die "programming error, exoticness not defined" if ! defined $exoticness;
      my %weights = ();
      for (my $i=0; $i<=11; $i++) {
        $weights{$i} = 1+$difficulty-$exoticness->[$i];
        if ($weights{$i}<0) {$weights{$i}=0}
        $weights{$i} = $weights{$i}**(3./$difficulty);
      }
      $note = $root + Util::random_array_element_weighted(\%weights);
      $exercise->{NOTE} = $note;
      $exercise->{ROOT} = Util::bring_within_octave($root,$exercise->{NOTE});
      $exercise->{NOTE_SOUND} = $activity->{TIMBRES}->{'melody'}->synthesize(CHORDS=>[$note,],RHYTHM=>[1,],VOLUME=>1./5.);
      $exercise->{CHORD_SOUND} = $activity->{TIMBRES}->{'accomp'}->synthesize(CHORDS=>[$chord,],RHYTHM=>[1,],VOLUME=>1./5.);
      $exercise->{ROOT_SOUND} = $activity->{TIMBRES}->{'accomp'}->synthesize(CHORDS=>[$root,],RHYTHM=>[1,],VOLUME=>1./5.);
      $exercise->{SOUND} = .5*($exercise->{NOTE_SOUND}+$exercise->{CHORD_SOUND});
      $activity->{DESCRIBE_CHORD} = {
                              'maj'=>'major','m'=>'minor','maj7'=>'major 7th','7'=>'dominant seventh',
                              'm7'=>'minor 7th','mmaj7'=>'tonic minor','m7b5'=>'half-diminished',
                            }->{$type};
      $activity->play($exercise->{SOUND});
    },
    replay=>sub {
      $activity->play($exercise->{SOUND});
    },
    thing=>sub {
      $activity->play($exercise->{NOTE_SOUND});
    },
    chord=>sub {
      $activity->play($exercise->{CHORD_SOUND});
    },
    root=>sub {
      $activity->play($exercise->{ROOT_SOUND});
    },
  });


  # subs that depend on the user interface:
  my $print_prompt = sub {
      my $state = shift;
      print "Enter ";
      if ($state eq 'waiting') {
        print "solfeg letter (".Util::list_crowell_names()."), or ";
      }
      print "? to hear it again, or n, c, or 1 to hear the note, chord, or root by itself,\n";
      if ($state eq 'incorrect') {
        print "or hit enter to go on, ";
      }
      print "or L to change the level, or q to quit this exercise\n";
    };
  my $set_level = sub {
    do {
      print "Enter a difficulty level from 1 to 10.\n";
      $difficulty = <STDIN>;
      chomp $difficulty;
    } while (!($difficulty>=1 && $difficulty<=10));
	};
  $activity->{FSM}->register({
	  waiting=>sub {&$print_prompt('waiting')},
		incorrect=>sub {&$print_prompt('incorrect')},
		level=>sub {&$set_level},
    describe=>sub {print $activity->{DESCRIBE_CHORD}; print "\n"},
  });

  $activity->{FSM}->start;

  for (;;) {
    my $state = $activity->{FSM}->state;
    last if ($state eq 'end');
    my $poss = $activity->{FSM}->list_transitions;
    my $input = <STDIN>;
    chomp $input;
    my $menu = {'?'=>'replay','1'=>,'root','n'=>'thing','c'=>'chord',''=>'next','L'=>'level','q'=>'quit'}->{$input};
    if ($menu && exists $poss->{$menu}) {
      $activity->{FSM}->transition($menu);
    }
    if (!$menu && $state eq 'waiting') {
      my $guess = Util::crowell_to_note($input);
      print "guess=".Util::note_to_crowell($guess)."\n";
      my $actual = ($exercise->{NOTE}-$exercise->{ROOT})%12;
      if ($guess==$actual) {
        print "correct\n";
        $activity->{FSM}->transition('next');
      }
      else {
        print "incorrect, the note was ".Util::note_to_crowell($actual)."\n";
        $activity->{FSM}->transition('incorrect');
      }
    }
  }

}

1;
