#!/usr/bin/perl

use strict;

use Audio::Data;
use Audio::Play;

use Synthesize;
use Music;
use FiniteStateMachine;
use Activity;
use TonalActivity;

package ExRelTonic;

sub do {
  my $server = shift;
  my $timbre = shift;
  my $timbre2 = shift;
  my $activity = TonalActivity->new(SERVER=>$server,TIMBRES=>{'accomp'=>$timbre,'melody'=>$timbre2},SAME_KEY_PROB=>.65);
  my $exercise = $activity->{EXERCISE};
  $activity->{FSM}->register({
    construct=>sub {
      my @scale = $activity->random_scale_as_array();
      my $note;
      for (;;) {
        $note = Util::random_array_element(@scale);
        $note+=Util::random_integer_in_range(-3,3) if rand()<.1; # may give note outside scale; wide range to avoid wierd distribution
        last if $note!=$activity->{NOTE};
      }
      my $timbre = $activity->{TIMBRES}->{'melody'};
      $note = Util::random_octave_transposition($note,$timbre->best);
      $exercise->{NOTE} = $note;
      $exercise->{TONIC} = Util::bring_within_octave($exercise->{TONIC},$exercise->{NOTE});
      $exercise->{NOTE_SOUND} = $timbre->synthesize(CHORDS=>[Chord->new($note)],RHYTHM=>[1,1],VOLUME=>1.);
      $activity->play_cadence();
      $activity->play($exercise->{NOTE_SOUND});
    },
    replay=>sub {
      $activity->play_cadence();
      $activity->play($exercise->{NOTE_SOUND});
    },
    thing=>sub {
      $activity->play($exercise->{NOTE_SOUND});
    },
  });

  # subs that depend on the user interface:
  my $print_prompt = sub {
      my $state = shift;
      print "enter ";
      if ($state eq 'waiting') {
        print "solfeg letter (".Util::list_crowell_names()."), or ";
      }
      print "? to hear it again, or n, c, or 1 to hear the note, cadence, or tonic by itself, ";
      if ($state eq 'incorrect') {
        print "or hit enter to go on, ";
      }
      print "or q to quit this exercise\n";
    };
  $activity->{FSM}->register({
	  waiting=>sub {&$print_prompt('waiting')},
		incorrect=>sub {&$print_prompt('incorrect')},
  });

  $activity->{FSM}->start;

  for (;;) {
    my $state = $activity->{FSM}->state;
    last if ($state eq 'end');
    my $poss = $activity->{FSM}->list_transitions;
    my $input = <STDIN>;
    chomp $input;
    my $menu = {'?'=>'replay','1'=>,'tonic','n'=>'thing','c'=>'cadence',''=>'next','q'=>'quit'}->{$input};
    if ($menu && exists $poss->{$menu}) {
      $activity->{FSM}->transition($menu);
    }
    if (!$menu && $state eq 'waiting') {
      my $guess = Util::crowell_to_note($input);
      print "guess=".Util::note_to_crowell($guess)."\n";
      my $actual = ($exercise->{NOTE}-$exercise->{TONIC})%12;
      if ($guess==$actual) {
        print "correct\n";
        $activity->{FSM}->transition('next');
      }
      else {
        print "incorrect, the note was ".Util::note_to_crowell($actual)."\n";
        $activity->{FSM}->transition('incorrect');
      }
    }
  }

}



1;
