#----------------------------------------------------------------
# Copyright (c) 2004 Benjamin Crowell, all rights reserved.
#
# This software is available under version 2 of the GPL license.
# The software is copyrighted, and you must agree to the
# license in order to have permission to copy it. The full
# text of the license is given in the file titled Copying.
#
#----------------------------------------------------------------


use strict;

use Music;
use Lily;
use Clone;

package Melody;
=head1 Melody class

Represents a melody, with a single voice (not harmonized).

=cut

=head2 random

Melody->random('/where/they/are');

=cut

sub random {
  my $class = shift;
  my $dir = shift;
  my $self = {};
  bless($self,$class);

  my $ly_file;
  do {
    do {
      $ly_file = Util::random_array_element(glob "$dir/*.ly")
    } while ($ly_file=~m/x\_/);
    $self->{DATA} = Lily::parse_melody($ly_file);
  } while (! defined $self->tonic);

  $self->{LY_FILE} = $ly_file;

  my @foo = (1) x ($self->length);
  $self->{DATA}->{volumes} = \@foo;

  $self->bring_tonic_close_to_midpoint_of_range;

  return $self;
}

sub synthesize {
  my $self = shift;
  my $timbre = shift;
  my $volume = shift;
  my $tempo = shift;
  my $foo = $self->{DATA}->{notes};
  return $timbre->synthesize(CHORDS=>$self->{DATA}->{notes},RHYTHM=>$self->{DATA}->{rhythm},PROPERTIES=>$self->{DATA}->{properties},
                             NOTE_VOLUME=>$self->{DATA}->{volumes},VOLUME=>$volume,TEMPO=>$tempo);
}

sub transpose {
  my $self = shift;
  my $transposition = shift;
  my $notes = $self->notes;
  my @notes = @$notes;
  @notes = Util::add_constant_to_array($transposition,@notes);
  $self->{DATA}->{notes} = \@notes;
  $self->{DATA}->{meta}->{tonic} += $transposition if defined $self->tonic;
}

sub mode {
  my $self = shift;
  return $self->{DATA}->{meta}->{mode};
}

sub volumes {
  my $self = shift;
  return $self->{DATA}->{volumes};
}

sub amplify {
  my $self = shift;
  my $amp = shift;
  my $vols = $self->volumes;
  for (my $i=0; $i<@$vols; $i++) {
    $vols->[$i] *= $amp;
  }
  $self->{DATA}->{volumes} = $vols;
}

sub notes {
  my $self = shift;
  return $self->{DATA}->{notes};
}

sub length {
  my $self = shift;
  my $notes = $self->notes;
  return scalar @$notes;
}

sub slice {
  my $self = shift;
  my $from = shift;
  my $to = shift;
  my $new = $self->clone;
  foreach my $thingie('notes','rhythm','bar','properties','letters_int','scale_degrees','volumes') {
    my $foo = $self->{DATA}->{$thingie};
    my @foo = @$foo;
    @foo = @foo[$from..$to];
    $new->{DATA}->{$thingie} = \@foo;
  }
  return $new;
}

sub lo_and_hi {
  my $self = shift;
  my $notes = $self->notes;
  return Util::array_lo_and_hi(@$notes);
}

sub midpoint_of_range {
  my $self = shift;
  my ($lo,$hi) = $self->lo_and_hi;
  return int(($lo+$hi)/2);
}

sub bring_tonic_close_to_midpoint_of_range {
  my $self = shift;
  my $tonic = $self->tonic;
  $tonic = Util::bring_within_octave($tonic,$self->midpoint_of_range);
  $self->{DATA}->{meta}->{tonic} = $tonic;  
}

sub tonic {
  my $self = shift;
  my $tonic = $self->{DATA}->{meta}->{tonic};
  return undef if !defined $tonic;
  return undef if $tonic==0;
  return $tonic;
}

sub description {
  my $self = shift;
  my $title = $self->{DATA}->{meta}->{header_title};
  my $composer = $self->{DATA}->{meta}->{header_composer};
  if (defined $title || defined $composer) {
    return "$composer, $title" if (defined $title && defined $composer);
    return $composer if ((!defined $title) && defined $composer);
    return $title if (defined $title && ! defined $composer);
  }
  else {
    return $self->{LY_FILE};
  }
}

sub tempo {
  my $self = shift;
 	my $tempo = 0.5;
 	$tempo = $self->{DATA}->{meta}->{tempo} if (exists $self->{DATA}->{meta}->{tempo});
  return $tempo;
}

sub timesig {
  my $self = shift;
 	my $timesig = 0.5;
 	$timesig = $self->{DATA}->{meta}->{time_signature} if exists $self->{DATA}->{meta}->{time_signature};
  return $timesig;
}


1;
