#!/usr/bin/perl

use strict;

use Audio::Data;
use Audio::Play;

use Synthesize;
use ExRelTonic;
use ExNoteChord;
use ExInterval;
use ExMelodyInterval;
use ExChord;
use Lily;
use Analyze;

my $server = Audio::Play->new;
my $instrument = 'p';
my $piano = Timbre->new("data/samples/$instrument","$instrument*.au");
if (! ref $piano) {die $piano}

$instrument = 'sax';
my $sax = Timbre->new("data/samples/$instrument","$instrument*.au");
if (! ref $sax) {die $sax}

my $timbre1 = $piano;
my $timbre2 = $sax;

my $rate = 44100;


my $tempo = .3;

for (;;) {
	print "Enter note number, or start,end\n";
  my $stuff = <STDIN>;
  chomp $stuff;
  my $start = $stuff;
  my $end = $start;
  if ($stuff=~m/(\d+),(\d+)/) {
    ($start,$end) = ($1,$2);
  }
  
  for (my $note=$start; $note<=$end; $note++) {
    print "note=$note\n";

    my $sound1 = $timbre1->synthesize(CHORDS=>[$note,],RHYTHM=>[1,],VOLUME=>1.,TEMPO=>$tempo);
    my $sound2 = $timbre2->synthesize(CHORDS=>[$note,],RHYTHM=>[1,],VOLUME=>0.35,TEMPO=>$tempo);

    get_err($sound1,$note,$rate,'piano');
    get_err($sound2,$note,$rate,'sax  ');

    my $sil = Audio::Data->new(rate=>$rate);
    $sil->silence(.4);

    my $sound = $sound1 . $sound2 . ($sound1+($sil.$sound2));

    $server->play($sound);
  }
}

sub get_err {
  my $sound = shift;
  my $note = shift;
  my $rate = shift;
  my $name = shift;
  my $expected_period = ($rate/440.)/(2.**(($note-69.)/12.)); # midi note 69 is A, 440 Hz, concert pitch
  my $expected_frequency = $rate/$expected_period;
  my $min = $expected_period/1.5;
  my $max = $expected_period*1.5;
  my $size = 2**16;
  my $period = Analyze::period_biggest_fft_peak($sound,0,2**16-1,$min,$max);
  my $frequency = $rate/$period;
  my $pitch_err = 1200.*log($frequency/$expected_frequency)/log(2.); # error in units of cents
  print "  $name  frequency=$frequency Hz, expected=$expected_frequency Hz, error=$pitch_err cents\n";
}
