#!/usr/bin/perl

use strict;

use Audio::Data;
use Audio::Play;

use Analyze;


my $server = Audio::Play->new;



my $instrument = 'sax';



my $fix_tuning = 1;
my $global_tuning_correction = 1.000; # how much do we think all pitches need to be adjusted? would be unnecessary except for the nonlinear fn below:
my $fix_tuning_sub = sub {
  my $x = shift; # proposed pitch correction is to multiply frequency by 1+x
  return 0. if abs($x)>0.01; # Either the pitch detection is confused, or it's the wrong note. Don't try to fix it.
  my $a = 0.0017; # Don't do corrections bigger than this. E.g., the extreme octaves on a piano are stretched a lot, and we don't want to undo that.
  return $a*($x/($a+abs($x)));
};
my $find_attack_sub = sub{my @a = @_; return Analyze::find_attack_simple(@a);};


my @wav_files = ();

foreach my $wav_file(<data/samples/$instrument/${instrument}*.wav>) {
  push @wav_files,$wav_file;
}


my $max_seconds = 5.; # seconds; sounds longer than this get truncated
my $rms_test_period = 0.1; # seconds; period after the attack over which to sample the rms loudness
my $desired_rms_after = 0.4; # scale rms to be this, during the rms_test_period

foreach my $wav_file(@wav_files) {
  my $au_file = $wav_file;
  $au_file =~ s/\.wav$/\_raw\.au/;
  my $processed_file = $wav_file;
  $processed_file =~ s/\.wav$/\.au/;
  $wav_file =~ m/(\d+)\.wav/;
  my $note = $1; # midi note number, 60=middle C
  if (! -e $au_file || (stat($wav_file))[9]>(stat($au_file))[9]) { # au file doesn't exist, or the wav file was modified after the au file was
    system("sox $wav_file $au_file")==0 or die "error converting to .au, $!";
    print "Converting $wav_file to $au_file\n";
  }
  my $sound = get_audio_from_au_file($au_file);
  my $n = $sound->samples;
  my $rate = $sound->rate;
  my $attack = &$find_attack_sub($sound);
  my ($dc,$rms_before) = Analyze::estimate_rms($sound,0,$attack);
  my ($junk,$rms_after) = Analyze::estimate_rms($sound,$attack,$attack+$rate*$rms_test_period);

  my $part = $sound->timerange($attack/$rate,$attack/$rate+0.5);
  print "note=$note\n";
  print "  rate=$rate Hz\n";
  print "  dc=".int(1000*$dc)."\n";
  print "  attack=".int((1000*$attack)/($rate))." ms, rms before=".int(1000*$rms_before).", rms_after=".int(1000*$rms_after)."\n";

  my $expected_period = ($rate/440.)/(2.**(($note-69.)/12.)); # midi note 69 is A, 440 Hz, concert pitch
  my $expected_frequency = $rate/$expected_period;
  my $min = $expected_period/1.5;
  my $max = $expected_period*1.5;
  my $size = 2**16;
  my $period = Analyze::period_biggest_fft_peak($sound,$attack,$attack+$size-1,$min,$max);
  my $frequency = $rate/$period;
  my $pitch_err = 1200.*log($frequency/$expected_frequency)/log(2.); # error in units of cents
  print "  frequency=$frequency Hz, expected=$expected_frequency Hz, error=$pitch_err cents\n";

  $sound = $sound->timerange($attack/$rate,$n/$rate);
  $sound = $sound - $dc;
  $sound = $sound *($desired_rms_after/$rms_after);
  if ($sound->samples >$max_seconds*$rate) {
    $sound = $sound->timerange(0.,$max_seconds);
  }

  print "playing...\n";
  $server->play($sound);
  print "...done\n";

  save_audio_to_au_file($processed_file,$sound);
  if ($fix_tuning) {

    my $cmd = sprintf "cp $processed_file temp.wav ; sox temp.wav $processed_file speed %8.5f",
                ($global_tuning_correction*(1.+&$fix_tuning_sub((1./$global_tuning_correction)*$expected_frequency/$frequency-1.)));
    print "$cmd\n";
    system($cmd)==0 or die "error, $!";
  }

  #last if $note>48;
}

sub save_audio_to_au_file {
  my $filename = shift;
  my $audio = shift;
  open(FH,">$filename");
  binmode FH;
  $audio->Save(*FH);
  close FH;
}

sub get_audio_from_au_file {
  my $filename = shift;
  my $audio = Audio::Data->new();
  open(FH,"<$filename");
  binmode FH;
  $audio->Load(*FH);
  close FH;
  return $audio;
}
